from django.conf import settings

NETCASH_TARGET_URL = 'https://gateway.netcash.co.za/vvonline/ccnetcash.asp'

NETCASH_USERNAME = settings.NETCASH_USERNAME
NETCASH_PASSWORD = settings.NETCASH_PASSWORD
NETCASH_PIN = settings.NETCASH_PIN
NETCASH_TERMINAL_NUMBER = settings.NETCASH_TERMINAL_NUMBER

# request.META key with client ip address
NETCASH_IP_HEADER = getattr(settings, 'NETCASH_IP_HEADER', 'REMOTE_ADDR')

# NETCASH_TEST_MODE = getattr(settings, 'NETCASH_TEST_MODE', False)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'NetcashGateway'
        db.create_table('netcash_netcashgateway', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(default='gateway', max_length=50)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50, db_index=True)),
            ('secret', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('netcash_ip', self.gf('django.db.models.fields.IPAddressField')(max_length=15, null=True, blank=True)),
        ))
        db.send_create_signal('netcash', ['NetcashGateway'])

        # Adding model 'NetcashOrder'
        db.create_table('netcash_netcashorder', (
            ('Reference', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('TransactionAccepted', self.gf('django.db.models.fields.NullBooleanField')(default=None, null=True, blank=True)),
            ('CardHolderIpAddr', self.gf('django.db.models.fields.IPAddressField')(max_length=15, null=True, blank=True)),
            ('Amount', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=12, decimal_places=2, blank=True)),
            ('Reason', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('RETC', self.gf('django.db.models.fields.CharField')(max_length=25, null=True, blank=True)),
            ('Extra1', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('Extra2', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('Extra3', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('gateway', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['netcash.NetcashGateway'], null=True, blank=True)),
        ))
        db.send_create_signal('netcash', ['NetcashOrder'])


    def backwards(self, orm):
        
        # Deleting model 'NetcashGateway'
        db.delete_table('netcash_netcashgateway')

        # Deleting model 'NetcashOrder'
        db.delete_table('netcash_netcashorder')


    models = {
        'netcash.netcashgateway': {
            'Meta': {'object_name': 'NetcashGateway'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'gateway'", 'max_length': '50'}),
            'netcash_ip': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'secret': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50', 'db_index': 'True'})
        },
        'netcash.netcashorder': {
            'Amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'CardHolderIpAddr': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'Extra1': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Extra2': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Extra3': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Meta': {'object_name': 'NetcashOrder'},
            'RETC': ('django.db.models.fields.CharField', [], {'max_length': '25', 'null': 'True', 'blank': 'True'}),
            'Reason': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'Reference': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'TransactionAccepted': ('django.db.models.fields.NullBooleanField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'gateway': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['netcash.NetcashGateway']", 'null': 'True', 'blank': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'})
        }
    }

    complete_apps = ['netcash']



from django.conf.urls.defaults import *
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'^netcash/', include('netcash.urls')),
)



from django.contrib import admin
from netcash.models import NetcashGateway, NetcashOrder

class NetcashGatewayAdmin(admin.ModelAdmin):
    list_display = ['name', 'data_url', 'accept_url', 'reject_url', 'netcash_ip']

class NetcashOrderAdmin(admin.ModelAdmin):
    list_display = ['Reference', 'created_at', 'updated_at', 'Amount',
                    'Reason', 'RETC', 'request_ip', 'TransactionAccepted', 'trusted']
    list_filter = ['TransactionAccepted', 'trusted']
    search_fields = ['Reference']
    date_hierarchy = 'created_at'

admin.site.register(NetcashGateway, NetcashGatewayAdmin)
admin.site.register(NetcashOrder, NetcashOrderAdmin)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Deleting field 'NetcashGateway.slug'
        db.delete_column('netcash_netcashgateway', 'slug')


    def backwards(self, orm):
        
        # We cannot add back in field 'NetcashGateway.slug'
        raise RuntimeError(
            "Cannot reverse this migration. 'NetcashGateway.slug' and its values cannot be restored.")


    models = {
        'netcash.netcashgateway': {
            'Meta': {'object_name': 'NetcashGateway'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'gateway'", 'max_length': '50'}),
            'netcash_ip': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'secret': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'netcash.netcashorder': {
            'Amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'CardHolderIpAddr': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'Extra1': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Extra2': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Extra3': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'Meta': {'object_name': 'NetcashOrder'},
            'RETC': ('django.db.models.fields.CharField', [], {'max_length': '25', 'null': 'True', 'blank': 'True'}),
            'Reason': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'Reference': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'TransactionAccepted': ('django.db.models.fields.NullBooleanField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'gateway': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['netcash.NetcashGateway']", 'null': 'True', 'blank': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'})
        }
    }

    complete_apps = ['netcash']



#coding: utf-8
import random
import string
from datetime import datetime
from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site

def full_url(link):
    current_site = Site.objects.get_current()
    return current_site.domain + link

def data_url(gateway):
    return full_url(reverse('netcash_data', args=[gateway.secret]))

def random_secret():
    return "".join([random.choice(string.letters) for x in range(0,40)])


class NetcashGateway(models.Model):
    """ Secure settings for payment gateway """

    name = models.CharField(max_length=50, default='gateway')
    secret = models.CharField(max_length=40, editable=False,
                              help_text='Data URL secret')

    netcash_ip = models.IPAddressField('Netcash server ip address', null=True, blank=True,
                    help_text='If specified, the requests to Data URL would '
                    'be only considered valid if they came from this ip')

    def data_url(self):
        return data_url(self)

    def accept_url(self):
        return full_url(reverse('netcash_accept'))

    def reject_url(self):
        return full_url(reverse('netcash_reject'))

    def save(self, *args, **kwargs):
        if not self.secret:
            self.secret = random_secret()

        return super(NetcashGateway, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = 'Netcash gateway'
        verbose_name_plural = 'Netcash gateway settings'


class NetcashOrder(models.Model):

    # field names are not pep-08 in order to match Netcash API docs

    Reference = models.AutoField(primary_key=True,
                          help_text='This is the unique reference that have '
                          'been sent to Netcash in the original request')

    TransactionAccepted = models.NullBooleanField(default=None)
    CardHolderIpAddr = models.IPAddressField(u'Cardholder ip address', null=True, blank=True)
    Amount = models.DecimalField(decimal_places=2, max_digits=12, null=True, blank=True)

    Reason = models.CharField(max_length=255, null=True, blank=True,
                       help_text='If the transaction failed this is the reason '
                       'returned for the failure')

    RETC = models.CharField(max_length=25, null=True, blank=True,
                     help_text = "This is a code returned by the system for "
                    "this transaction. Should you need to make an "
                    "enquiry you will use this number to reference "
                    "this transaction")

    Extra1 = models.CharField(null=True, blank=True, max_length = 50)
    Extra2 = models.CharField(null=True, blank=True, max_length = 50)
    Extra3 = models.CharField(null=True, blank=True, max_length = 50)

    # utility fields
    created_at = models.DateTimeField(default=datetime.now)
    updated_at = models.DateTimeField(default=datetime.now)
    gateway = models.ForeignKey(NetcashGateway, null=True, blank=True)
    request_ip = models.IPAddressField(null=True, blank=True)
    debug_info = models.CharField(max_length=255, null=True, blank=True)
    trusted = models.NullBooleanField(default=None)

    def save(self, *args, **kwargs):
        self.updated_at = datetime.now()
        return super(NetcashOrder, self).save(*args, **kwargs)

    def __unicode__(self):
        return u'NetCash #%s (%s)' % (self.pk, self.created_at)

    class Meta:
        verbose_name = 'NetCash order'
        verbose_name_plural = 'NetCash orders'



# Django settings for merchant_api_tests project.
import os, sys
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
join = lambda p: os.path.abspath(os.path.join(PROJECT_ROOT, p))

# use netcash from the folder above, not the installed version
sys.path.insert(0, join('..'))

# ===== netcash settings ====

NETCASH_USERNAME = 'testuser'
NETCASH_PASSWORD = 'testpass'
NETCASH_PIN = '654321'
NETCASH_TERMINAL_NUMBER = '0291'

# ===========================

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = ()
MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': join('db.sqlite'),                      # Or path to database file if using sqlite3.
    }
}

TIME_ZONE = 'America/Chicago'
LANGUAGE_CODE = 'en-us'
SITE_ID = 1
USE_I18N = True
USE_L10N = True
MEDIA_ROOT = join('media')
MEDIA_URL = '/media/'
ADMIN_MEDIA_PREFIX = '/media/admin/'
SECRET_KEY = '5mcs97ar-(nnxhfk67290+0^sr!e(ax=x$2-!8dqy25ff-l1*a='
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'urls'
TEMPLATE_DIRS = (join('templates'),)

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'netcash',
]

# test migrations if South is available
try:
    import south
    if 'south' not in INSTALLED_APPS:
        INSTALLED_APPS += ['south']
except ImportError:
    pass



from django.test import TestCase
from django.core.urlresolvers import reverse

from netcash.models import NetcashGateway, NetcashOrder
from netcash.forms import NetcashForm
import netcash.signals

class NetcashTest(TestCase):

    def setUp(self):
        self.gateway = NetcashGateway.objects.all()[0]
        self.gateway.netcash_ip = '127.0.0.1'
        self.gateway.save()

        # reload the gateway
        self.gateway = NetcashGateway.objects.all()[0]

        self.gateway_url = reverse('netcash_data', args = [self.gateway.secret])

    def test_invalid_data_request(self):
        response = self.client.post(self.gateway_url, {})
        self.assertEqual(response.status_code, 404)
        self.assertEqual(NetcashOrder.objects.count(), 0)

    def test_invalid_gateway(self):
        response = self.client.post(self.gateway_url.rstrip('/')+'123/', {})
        self.assertEqual(response.status_code, 404)
        self.assertEqual(NetcashOrder.objects.count(), 0)

    def test_data_request(self):
        # signal handler
        def data_handler(sender, order, **kwargs):
            data_handler.called = True
            data_handler.order = order
        data_handler.called = False
        data_handler.order = None

        netcash.signals.data.connect(data_handler)

        # this will create an order
        form = NetcashForm(initial={
            'p3': 'description',
            'p4': 100,
        })
        reference = str(form.order.pk)
        self.assertEqual(form.order.trusted, None)

        self.assertFalse(data_handler.called)

        # ... user posts the form and pays the purchase on netcash.co.za site
        # ... then Netcash sends a notification to our Data URL

        response = self.client.post(self.gateway_url, {
           'TransactionAccepted': 'true',
           'Reference': reference,
           'RETC': '123',
           'Reason': '',
           'Amount': '100'
        })

        self.assertEqual(response.status_code, 200)

        order = NetcashOrder.objects.get(pk=reference)
        self.assertEqual(order.TransactionAccepted, True)
        self.assertEqual(order.Amount, 100)
        self.assertEqual(order.RETC, '123')
        self.assertEqual(order.gateway, self.gateway)
        self.assertEqual(order.request_ip, '127.0.0.1')
        self.assertEqual(order.trusted, True)

        self.assertTrue(data_handler.called)
        self.assertEqual(data_handler.order, order)




#coding: utf-8
from django.http import HttpResponse, Http404
from django.views.generic.simple import direct_to_template
from django.shortcuts import get_object_or_404

try:
    from django.views.decorators.csrf import csrf_exempt
except ImportError: # django < 1.2
    from django.contrib.csrf.middleware import csrf_exempt

from netcash.forms import DataHandlerForm
from netcash.models import NetcashOrder, NetcashGateway
from netcash import signals
from netcash.conf import NETCASH_IP_HEADER

@csrf_exempt
def data_handler(request, secret):
    """
    Data URL handler.

    This view is secured by 40-byte random secret string in the URL and
    the ip address limiting.

    On successful access 'netcash.signals.data' signal is sent.
    Orders should be processed in signal handler.
    """
    gateway = get_object_or_404(NetcashGateway, secret=secret)
    ip = request.META.get(NETCASH_IP_HEADER, None)
    reference = request.POST.get('Reference', None)
    order = get_object_or_404(NetcashOrder, pk=reference)

    if gateway.netcash_ip:
        # deny query if it come from untrusted ip
        if ip != gateway.netcash_ip:
            order.request_ip = ip
            order.debug_info = 'untrusted ip: %s != %s' % (gateway.netcash_ip, ip)
            order.trusted = False
            order.save()
            raise Http404

    form = DataHandlerForm(request.POST, instance = order)
    if form.is_valid():
        order = form.save(commit=False)
        order.request_ip = ip
        order.gateway = gateway
        order.debug_info = request.raw_post_data
        order.trusted = True
        order.save()
        signals.data.send(sender = data_handler, order=order)
        return HttpResponse()

    errors = form.plain_errors()[:255]
    order.debug_info = errors
    order.trusted = False
    order.save()
    return HttpResponse(errors)


@csrf_exempt
def result_handler(request, template_name, extra_context=None):
    """
    Accept or Reject URL handler.

    There is no signing in Netcash protocol and these URLs are
    visible to user so processing orders on Accept/Reject URL access is very
    insecure and must be avoided. That's why django-netcash just renders a
    template and doesn't provide a hook for order processing here.
    """
    reference = request.REQUEST.get('Reference', False)
    order = get_object_or_404(NetcashOrder, pk=reference)
    context = {'order': order}
    context.update(extra_context or {})
    return direct_to_template(request, template_name, context)




#!/usr/bin/env python
from distutils.core import setup

setup(
    name='django-netcash',
    version='0.1.8',
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['netcash', 'netcash.migrations'],

    url='http://bitbucket.org/kmike/django-netcash/',
    download_url = 'http://bitbucket.org/kmike/django-netcash/get/tip.gz',
    license = 'MIT license',
    description = 'A pluggable Django application for integrating netcash.co.za payment system.',
    long_description = open('README.rst').read().decode('utf8'),

    classifiers=(
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)



#coding: utf-8
from django.conf.urls.defaults import *

urlpatterns = patterns('netcash.views',
    url(
          r'^data/(?P<secret>.+)/$',
          'data_handler',
          name='netcash_data'
    ),
    url(
          r'^accept/$',
          'result_handler',
          {'template_name': 'netcash/accept.html'},
          name='netcash_accept'
    ),
    url(
          r'^reject/$',
          'result_handler',
          {'template_name': 'netcash/reject.html'},
          name='netcash_reject'
    ),
)



from django import forms

from netcash.conf import *
from netcash.models import NetcashOrder

class HiddenForm(forms.Form):
    """ A form with all fields hidden """
    def __init__(self, *args, **kwargs):
        super(HiddenForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].widget = forms.HiddenInput()


class NetcashForm(HiddenForm):
    """ NetCash helper form.
    It is not for validating data.
    It can be used to output html. """

    target = NETCASH_TARGET_URL

    # these params are handled automatically
    m_1 = forms.CharField(max_length = 50, initial = NETCASH_USERNAME)
    m_2 = forms.CharField(max_length = 50, initial = NETCASH_PASSWORD)
    m_3 = forms.CharField(max_length = 50, initial = NETCASH_PIN)
    p1 = forms.CharField(max_length = 50, initial = NETCASH_TERMINAL_NUMBER)
    p2 = forms.CharField(max_length = 25)

    # The description of the goods sent for payment
    p3 = forms.CharField(max_length = 50)

    # Transactional Amount that is to be settled to the Card
    p4 = forms.DecimalField(min_value=0, decimal_places=2, max_digits=8)

    # Cancel Button URL. This is the URL the client will be directed
    # to when at anytime the client clicks the cancel button.
    p10 = forms.URLField(max_length=255, verify_exists=False, required=False)

    # 'Y' will display the budget option in the Gateway popup
    # 'N' will not display the budget option in the Gateway popup
    Budget = forms.ChoiceField(choices=('Y', 'N'), initial='N')

    # Extra fields that can contain any data that you require
    # back from the gateway once the settlement has been done
    m_4 = forms.CharField(max_length=50, required=False)
    m_5 = forms.CharField(max_length=50, required=False)
    m_6 = forms.CharField(max_length=50, required=False)

    # Card holders email address should you want an email sent to the cardholder
    m_9 = forms.CharField(max_length=100, required=False)

    # Any text sent in this parameter is returned to the Accept and Reject
    # return URL's. This is usually used with basket products like
    # OSCommerce and VirtueMart
    m_10 = forms.CharField(max_length=100, required=False)

    def __init__(self, *args, **kwargs):
        super(NetcashForm, self).__init__(*args, **kwargs)

        # new order reference number is issued each time form is instantiated
        self.order = NetcashOrder.objects.create()
        self.fields['p2'].initial = self.order.pk


class DataHandlerForm(forms.ModelForm):

    # This fields are named differently on data page for some reason
    m_4 = forms.CharField(max_length=50, required=False)
    m_5 = forms.CharField(max_length=50, required=False)
    m_6 = forms.CharField(max_length=50, required=False)

    TransactionAccepted = forms.CharField()

    def plain_errors(self):
        ''' plain error list (without the html) '''
        return '|'.join(["%s: %s" % (k, (v[0])) for k, v in self.errors.items()])

    def clean_TransactionAccepted(self):
        accepted = self.cleaned_data['TransactionAccepted']
        if accepted == 'true':
            return True
        elif accepted == 'false':
            return False
        else:
            raise forms.ValidationError('Invalid value')

    def save(self, *args, **kwargs):
        self.cleaned_data['Extra1'] = self.cleaned_data['m_4']
        self.cleaned_data['Extra2'] = self.cleaned_data['m_5']
        self.cleaned_data['Extra3'] = self.cleaned_data['m_6']
        return super(DataHandlerForm, self).save(*args, **kwargs)

    class Meta:
        model = NetcashOrder
        exclude = ['created_at', 'updated_at']
